﻿// Copyright 2009 Oliver Sturm <oliver@sturmnet.org> All rights reserved. 

#light

// using namespaces
open System
open System.Text
open System.Windows.Forms

open Helpers

// assigning values
// basic types: (s)byte, (u)int16, (u)int32, (u)int64, 
// string, single, double, char, (u)nativeint, bool, decimal, unit
// also (u)int8, int, float32, float

// int
let a = 5
// string
let s = "hi"
// bool
let t = a = 5
// bigint
let bi = 100I

// Tuples

Output.banner "Tuples"

// int*int
let c1 = (10, 20)
// int*string*int
let c2 = (30, "text", 40)    

// picking pairs apart
printfn "first: %d" (fst c1)
printfn "snd: %d" (snd c1)

// picking any tuple apart
let part1, part2, part3 = c2
printfn "part1: %d, part2: %s, part3: %d" part1 part2 part3

// StringBuilder - a .NET type
let sb = StringBuilder()
// Form - using new because it implements IDisposable
let f = new Form()

Output.banner "Option types"

// string option
let o1 = Some("Text")
// 'a option - generic
let o2 = None

// int list
let l1 = [10;20;30]
// (string * string option) list
let l2 = 
    [
        ("Willy", Some("Mercedes"));
        ("Harry", Some("Vauxhall"));
        ("Anna", None)
    ]

// empty list
let emptyList = []

// "cons" (construct, basically prepend) for lists
let l3 = 90 :: l1

// append - concatenate
let l4 = l3 @ [110; 130]

// match expressions
let extractedText = 
    match o1 with
    | Some(s) -> s
    | None -> "No text"

printfn "Text found: %s" extractedText

Output.banner "Lists"

let showPartsOfList l = 
    match l with
    | [] -> printfn "List is empty"
    | hd :: tl -> printfn "Head: %A, Tail: %A" hd tl

showPartsOfList l1

// mutable types

Output.banner "Mutable types"

// mutable variables can't be captured in closures
let mutable mi1 = 5
mi1 <- 10

// ref cells can be part of closures
let ir1 = ref 10
ir1 := 42
// getting to the values
printfn "%d" ir1.Value
printfn "%d" !ir1

// int array - content is mutable!
let ar = [| 5; 3; 10 |]
printfn "%d" ar.[2]
ar.[1] <- 42
printfn "%A" ar

// creating functions

Output.banner "Functions"

// int -> int -> int
let add x y = x + y

let mult x y = 
    x * y

printfn "%d" (add 10 20)
printfn "%d" (mult 3 5)

// Option types
let checkOptionVal(o: 'a option) = 
    if o.IsSome then
        printfn "val is Some(%A)" o.Value
    else
        printfn "val is None"
        
checkOptionVal(o1)
checkOptionVal(o2)

// int -> bool
let isPrime x =
    let mutable result = true 
    for div in 2..(x/2) do
        if x % div = 0 then
            result <- false
    result

printfn "%b" (isPrime 13)
printfn "%b" (isPrime 10)
printfn "%b" (isPrime 47)

