﻿#light

type Person(name:string)=
    member x.Name=name
    
    override x.ToString() = name
    
open System
open System.Drawing
open System.Windows.Forms

type MyForm(data: Person list) as myform =
    inherit Form(
        Text="Filtered List",
        ClientSize=Size(660, 360),
        Font=new Font("Microsoft Sans Serif", float32(18), FontStyle.Regular, GraphicsUnit.Point, byte(0)))
        
    let displayListBox = 
        new ListBox(Location=Point(16, 16), Size=Size(440, 320))
    let filterTextBox = 
        new TextBox(Location=Point(470, 16), Size=Size(175, 35))
    let button = 
        new Button(Text="Filter", Location=Point(470, 65), Size=Size(175,50))
      
    do myform.InitializeComponent()
    do myform.InitData data

    member x.InitializeComponent() = 
        base.Controls.Add(displayListBox)
        base.Controls.Add(filterTextBox)
        base.Controls.Add(button)

    member x.InitData data = 
        let updateUI() = 
            displayListBox.DataSource <- (x.GetFilteredList data (x.GetFilterString()) :> System.Collections.Generic.IList<Person>)
        button.Click.AddHandler(new EventHandler(fun s e -> updateUI()))
        updateUI()
    
    member x.GetFilterString() = filterTextBox.Text
    
    member x.GetFilteredList data filter =
        match filter with 
        | "" -> new System.Collections.Generic.List<Person>(data)
        | _ -> new System.Collections.Generic.List<Person>(Seq.filter (fun (p:Person) -> p.Name.Contains(filter)) data)
    
let form = new MyForm(
                [
                    Person("Anna");
                    Person("Chris");
                    Person("Willy");
                    Person("Hugh");
                    Person("Steve");
                    Person("Betty");
                    Person("Carla");
                    Person("John");
                    Person("Pete");
                    Person("Susan")
                ])
        
Application.Run(form)
