﻿#light

type Person(name:string)=
    member x.Name=name
    
    override x.ToString() = name
    
open System
open System.Drawing
open System.Windows.Forms

type MyForm() as myform =
    inherit Form(
        Text="Filtered List",
        ClientSize=Size(660, 360),
        Font=new Font("Microsoft Sans Serif", float32(18), FontStyle.Regular, GraphicsUnit.Point, byte(0)))
        
    let displayListBox = 
        new ListBox(Location=Point(16, 16), Size=Size(440, 320))
    let filterTextBox = 
        new TextBox(Location=Point(470, 16), Size=Size(175, 35))
    let button = 
        new Button(Text="Filter", Location=Point(470, 65), Size=Size(175,50))
      
    let maxPeople = 10
    let mutable people: Person array = Array.zero_create maxPeople
    let mutable displayPeople: Person array = Array.zero_create maxPeople
    let mutable displayCount = 0
       
    do myform.InitializeComponent()
    do myform.InitData()
    do myform.UpdateUI()

    member x.InitializeComponent() = 
        base.Controls.Add(displayListBox)
        base.Controls.Add(filterTextBox)
        base.Controls.Add(button)
        button.Click.AddHandler(new EventHandler(fun s e -> x.ButtonClick(s, e)))

    member x.InitData() = 
        people.[0] <- Person("Anna")
        people.[1] <- Person("Chris")
        people.[2] <- Person("Willy")
        people.[3] <- Person("Hugh")
        people.[4] <- Person("Steve")
        people.[5] <- Person("Betty")
        people.[6] <- Person("Carla")
        people.[7] <- Person("John")
        people.[8] <- Person("Pete")
        people.[9] <- Person("Susan")
        x.ResetDisplayPeople()
        
    member x.ResetDisplayPeople() = 
        displayCount <- people.Length
        for i in 0..(displayCount - 1) do
            displayPeople.[i] <- people.[i]
            
    member x.UpdateUI() = 
        displayListBox.Items.Clear()
        for i in 0..(displayCount - 1) do
            displayListBox.Items.Add displayPeople.[i] |> ignore
    
    member x.ButtonClick(s, e) = 
        let filter = filterTextBox.Text
        if filter <> "" then
            x.FilterData filter
        else
            x.ResetDisplayPeople()
        x.UpdateUI()
   
    member x.FilterData filter = 
        let mutable filteredCount = 0
        for i in 0..(maxPeople-1) do
            if people.[i].Name.Contains(filter) then 
                displayPeople.[filteredCount] <- people.[i]
                filteredCount <- filteredCount + 1
        displayCount <- filteredCount
        
let form = new MyForm()
Application.Run(form)
